@@ -20,7 +20,7 @@ import libtorrent as lt
 from six import int2byte, text_type
 
 from twisted.internet import reactor
-from twisted.internet.defer import CancelledError, Deferred, succeed
+from twisted.internet.defer import CancelledError, Deferred, TimeoutError, succeed
 from twisted.internet.task import LoopingCall
 from twisted.python.failure import Failure
 
@@ -132,7 +132,6 @@ class LibtorrentDownloadImpl(TaskManager):
 
         # To be able to return the progress of a stopped torrent, how far it got.
         self.progressbeforestop = 0.0
-        self.filepieceranges = []
 
         # Libtorrent session manager, can be None at this point as the core could have
         # not been started. Will set in create_engine wrapper
@@ -222,7 +221,6 @@ class LibtorrentDownloadImpl(TaskManager):
 
                 if not isinstance(self.tdef, TorrentDefNoMetainfo):
                     self.set_corrected_infoname()
-                    self.set_filepieceranges()
 
                 self._logger.debug(u"setup: %s", hexlify(self.tdef.get_infohash()))
 
@@ -592,7 +590,6 @@ class LibtorrentDownloadImpl(TaskManager):
 
         self.orig_files = [ensure_text(torrent_file.path) for torrent_file in torrent_files]
         self.set_corrected_infoname()
-        self.set_filepieceranges()
         self.set_selected_files()
 
         self.checkpoint()
@@ -801,10 +798,10 @@ class LibtorrentDownloadImpl(TaskManager):
         return DownloadState(self, self.lt_status, self.error, vod)
 
     def _on_resume_err(self, failure):
-        failure.trap(CancelledError, SaveResumeDataError)
+        failure.trap(CancelledError, SaveResumeDataError, TimeoutError)
         self._logger.warning("Resume data failed to save: %s", failure.getErrorMessage())
 
-    def save_resume_data(self):
+    def save_resume_data(self, timeout=10):
         """
         Save the resume data of a download. This method returns a deferred that fires when the resume data is available.
         Note that this method only calls save_resume_data once on subsequent calls.
@@ -814,8 +811,7 @@ class LibtorrentDownloadImpl(TaskManager):
 
         defer_resume = Deferred()
         defer_resume.addErrback(self._on_resume_err)
-        defer_resume.addTimeout(10, reactor, onTimeoutCancel=lambda: self.on_save_resume_data_failed_alert(
-            type('anonymous_alert', (object,), dict(msg='Timed out'))))
+        defer_resume.addTimeout(timeout, reactor)
 
         self.deferreds_resume.append(defer_resume)
 
@@ -1003,13 +999,6 @@ class LibtorrentDownloadImpl(TaskManager):
         """ Returns the file to which the downloaded content is saved. """
         return os.path.join(self.config.get_dest_dir(), self.correctedinfoname)
 
-    def set_filepieceranges(self):
-        """ Determine which file maps to which piece ranges for progress info """
-        self._logger.debug("LibtorrentDownloadImpl: set_filepieceranges: %s", self.config.get_selected_files())
-
-        metainfo = self.tdef.get_metainfo()
-        self.filepieceranges = maketorrent.get_length_filepieceranges_from_metainfo(metainfo, [])[1]
-
     def restart(self):
         """ Restart the Download """
         self.config.set_user_stopped(False)
