@@ -4,7 +4,7 @@ import hashlib
 import os
 import sys
 import time
-from binascii import hexlify, unhexlify
+from binascii import unhexlify
 from collections import Counter
 from distutils.version import LooseVersion
 
@@ -15,19 +15,36 @@ from ipv8.messaging.anonymization.caches import CreateRequestCache
 from ipv8.messaging.anonymization.community import message_to_payload
 from ipv8.messaging.anonymization.hidden_services import HiddenTunnelCommunity
 from ipv8.messaging.anonymization.payload import LinkedE2EPayload, NO_CRYPTO_PACKETS
-from ipv8.messaging.anonymization.tunnel import CIRCUIT_STATE_CLOSING, CIRCUIT_STATE_READY, \
-                                                               CIRCUIT_TYPE_DATA, CIRCUIT_TYPE_IP_SEEDER,\
-                                                               CIRCUIT_TYPE_RP_DOWNLOADER, CIRCUIT_TYPE_RP_SEEDER,\
-                                                               EXIT_NODE, PEER_FLAG_EXIT_ANY, RelayRoute
+from ipv8.messaging.anonymization.tunnel import (
+    CIRCUIT_STATE_CLOSING,
+    CIRCUIT_STATE_READY,
+    CIRCUIT_TYPE_DATA,
+    CIRCUIT_TYPE_IP_SEEDER,
+    CIRCUIT_TYPE_RP_DOWNLOADER,
+    CIRCUIT_TYPE_RP_SEEDER,
+    EXIT_NODE,
+    PEER_FLAG_EXIT_ANY,
+    RelayRoute,
+)
 from ipv8.peer import Peer
 from ipv8.peerdiscovery.network import Network
 
 from twisted.internet.defer import Deferred, inlineCallbacks, succeed
 
 from Tribler.Core.Socks5.server import Socks5Server
-from Tribler.Core.simpledefs import (DLSTATUS_DOWNLOADING, DLSTATUS_METADATA, DLSTATUS_SEEDING, DLSTATUS_STOPPED,
-                                     NTFY_CREATED, NTFY_EXTENDED, NTFY_IP_RECREATE, NTFY_JOINED, NTFY_REMOVE,
-                                     NTFY_TUNNEL)
+from Tribler.Core.Utilities.unicode import hexlify
+from Tribler.Core.simpledefs import (
+    DLSTATUS_DOWNLOADING,
+    DLSTATUS_METADATA,
+    DLSTATUS_SEEDING,
+    DLSTATUS_STOPPED,
+    NTFY_CREATED,
+    NTFY_EXTENDED,
+    NTFY_IP_RECREATE,
+    NTFY_JOINED,
+    NTFY_REMOVE,
+    NTFY_TUNNEL,
+)
 from Tribler.community.triblertunnel.caches import BalanceRequestCache
 from Tribler.community.triblertunnel.discovery import GoldenRatioStrategy
 from Tribler.community.triblertunnel.dispatcher import TunnelDispatcher
@@ -219,10 +236,10 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
 
         def on_transaction_completed(blocks):
             # Send the next payout
-            if blocks and payload.circuit_id in self.relay_from_to and block.transaction['down'] > payload.base_amount:
+            if blocks and payload.circuit_id in self.relay_from_to and block.transaction[b'down'] > payload.base_amount:
                 relay = self.relay_from_to[payload.circuit_id]
                 self._logger.info("Sending next payout to peer %s", relay.peer)
-                self.do_payout(relay.peer, relay.circuit_id, block.transaction['down'] - payload.base_amount * 2,
+                self.do_payout(relay.peer, relay.circuit_id, block.transaction[b'down'] - payload.base_amount * 2,
                                payload.base_amount)
 
         block = TriblerBandwidthBlock.from_payload(payload, self.serializer)
@@ -261,7 +278,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
 
         # Get the latest block
         latest_block = self.bandwidth_wallet.trustchain.persistence.get_latest(self.my_peer.public_key.key_to_bin(),
-                                                                               block_type='tribler_bandwidth')
+                                                                               block_type=b'tribler_bandwidth')
         if not latest_block:
             latest_block = TriblerBandwidthBlock()
         latest_block.public_key = EMPTY_PK  # We hide the public key
@@ -286,7 +303,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
             self.on_token_balance(payload.circuit_id, 0)
         else:
             self.on_token_balance(payload.circuit_id,
-                                  block.transaction["total_up"] - block.transaction["total_down"])
+                                  block.transaction[b"total_up"] - block.transaction[b"total_down"])
 
     def on_relay_balance_response_cell(self, source_address, data, _):
         payload = self._ez_unpack_noauth(BalanceResponsePayload, data, global_time=False)
@@ -301,7 +318,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
                                BalanceResponsePayload.from_half_block(block, cache.to_circuit_id))
 
     def readd_bittorrent_peers(self):
-        for torrent, peers in self.bittorrent_peers.items():
+        for torrent, peers in list(self.bittorrent_peers.items()):
             infohash = hexlify(torrent.tdef.get_infohash())
             for peer in peers:
                 self.logger.info("Re-adding peer %s to torrent %s", peer, infohash)
@@ -331,8 +348,8 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
         self.logger.info("Sending payout of %d (base: %d) to %s (cid: %s)", amount, base_amount, peer, circuit_id)
 
         block = TriblerBandwidthBlock.create(
-            'tribler_bandwidth',
-            {'up': 0, 'down': amount},
+            b'tribler_bandwidth',
+            {b'up': 0, b'down': amount},
             self.bandwidth_wallet.trustchain.persistence,
             self.my_peer.public_key.key_to_bin(),
             link_pk=peer.public_key.key_to_bin())
@@ -393,7 +410,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
                 if self.tribler_session and self.tribler_session.config.get_libtorrent_enabled() else None
             if ltmgr:
                 for d, s in ltmgr.torrents.values():
-                    if s == ltmgr.get_session(d.get_hops()):
+                    if s == ltmgr.get_session(d.config.get_hops()):
                         d.get_handle().addCallback(lambda handle, download=d:
                                                    self.update_torrent(affected_peers, handle, download))
 
@@ -456,7 +473,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
 
         for ds in dslist:
             download = ds.get_download()
-            hop_count = download.get_hops()
+            hop_count = download.config.get_hops()
             if hop_count > 0:
                 # Convert the real infohash to the infohash used for looking up introduction points
                 real_info_hash = download.get_def().get_infohash()
@@ -482,7 +499,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
                                                self.settings.max_circuits)
                                 for hop_count, download_count in active_downloads_per_hop.items()}
 
-        ip_counter = Counter([c.info_hash for c in self.circuits.values() if c.ctype == CIRCUIT_TYPE_IP_SEEDER])
+        ip_counter = Counter([c.info_hash for c in list(self.circuits.values()) if c.ctype == CIRCUIT_TYPE_IP_SEEDER])
         for info_hash in set(list(new_states) + list(self.download_states)):
             new_state = new_states.get(info_hash, None)
             old_state = self.download_states.get(info_hash, None)
@@ -524,10 +541,11 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
             if LooseVersion(self.tribler_session.lm.ltmgr.get_libtorrent_version()) < LooseVersion("1.2.0"):
                 download.add_peer(('1.1.1.1', 1024))
             else:
-                hops = download.get_hops()
+                hops = download.config.get_hops()
                 lt_listen_port = self.tribler_session.lm.ltmgr.get_session(hops).listen_port()
                 for session in self.socks_servers[hops - 1].sessions:
-                    session.get_udp_socket().remote_udp_address = ("127.0.0.1", lt_listen_port)
+                    if session.get_udp_socket():
+                        session.get_udp_socket().remote_udp_address = ("127.0.0.1", lt_listen_port)
         super(TriblerTunnelCommunity, self).create_introduction_point(info_hash, amount)
 
     def on_linked_e2e(self, source_address, data, circuit_id):
@@ -554,7 +572,7 @@ class TriblerTunnelCommunity(HiddenTunnelCommunity):
         super(TriblerTunnelCommunity, self).unload()
 
     def get_lookup_info_hash(self, info_hash):
-        return hashlib.sha1(b'tribler anonymous download' + hexlify(info_hash)).digest()
+        return hashlib.sha1(b'tribler anonymous download' + hexlify(info_hash).encode('utf-8')).digest()
 
 
 class TriblerTunnelTestnetCommunity(TriblerTunnelCommunity):
