@@ -103,6 +103,9 @@ class TestBase(unittest.TestCase):
             # If we were to hard exit here (through os._exit) we would lose this additional information.
             import signal
             os.kill(os.getpid(), signal.SIGINT)
+            # But sometimes it just flat out refuses to die (sys.exit will also not work in this case).
+            # So we double kill ourselves:
+            os._exit(1)  # pylint: disable=W0212
         t = threading.Thread(target=check_twisted)
         t.daemon = True
         t.start()
