@@ -1,16 +1,45 @@
-from collections import deque, namedtuple
+from __future__ import absolute_import
+
 import logging
+from collections import deque, namedtuple
 from threading import RLock
 from time import time
 
-from PyQt5.QtCore import QUrl, pyqtSignal, QIODevice, QBuffer, QObject
+from PyQt5.QtCore import QBuffer, QIODevice, QObject, QUrl, pyqtSignal
 from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkRequest
 
+from six import string_types, text_type
+from six.moves import xrange
+from six.moves.urllib.parse import quote_plus
+
 import Tribler.Core.Utilities.json_util as json
-from TriblerGUI.defs import BUTTON_TYPE_NORMAL, DEFAULT_API_PORT, DEFAULT_API_PROTOCOL, DEFAULT_API_HOST
+
+from TriblerGUI.defs import BUTTON_TYPE_NORMAL, DEFAULT_API_HOST, DEFAULT_API_PORT, DEFAULT_API_PROTOCOL
 from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
 
 
+def tribler_urlencode(data):
+    # Convert all keys and values in the data to utf-8 unicode strings
+    utf8_items = []
+    for key, value in data.items():
+        if isinstance(value, list):
+            utf8_items.extend([tribler_urlencode_single(key, list_item) for list_item in value if value])
+        else:
+            utf8_items.append(tribler_urlencode_single(key, value))
+
+    data = "&".join(utf8_items)
+    return data
+
+
+def tribler_urlencode_single(key, value):
+    utf8_key = quote_plus(text_type(key).encode('utf-8'))
+    # Convert bool values to ints
+    if isinstance(value, bool):
+        value = int(value)
+    utf8_value = quote_plus(text_type(value).encode('utf-8'))
+    return "%s=%s" % (utf8_key, utf8_value)
+
+
 class QueuePriorityEnum(object):
     """
     Enum for HTTP request priority.
@@ -45,7 +74,7 @@ class RequestQueue(object):
         self.medium_queue = []
         self.low_queue = []
 
-        self.lock = RLock() # Don't allow asynchronous access to the queue
+        self.lock = RLock()  # Don't allow asynchronous access to the queue
 
     def parse_queue(self):
         """
@@ -115,7 +144,7 @@ class RequestQueue(object):
                 self.high_queue.append(queue_item)
             else:
                 # Get the last item of the queue
-                last_item = self.high_queue.pop(self.max_outstanding -1)
+                last_item = self.high_queue.pop(self.max_outstanding - 1)
                 # Add the original queue_item to the front of the queue
                 self.high_queue.insert(0, queue_item)
                 # reduce the priority of last_item and try to put in medium queue
@@ -224,9 +253,8 @@ class TriblerRequestManager(QObject):
     received_json = pyqtSignal(object, int)
     received_file = pyqtSignal(object)
 
-    def __init__(self, window=None):
+    def __init__(self):
         QObject.__init__(self)
-        self.window = window
         self.reply = None
         self.status_code = -1
         self.on_cancel = lambda: None
@@ -234,31 +262,42 @@ class TriblerRequestManager(QObject):
     def set_reply_handle(self, reply):
         self.reply = reply
 
-    def perform_request(self, endpoint, read_callback, data="", method='GET', capture_errors=True,
-                        priority=QueuePriorityEnum.CRITICAL, on_cancel=lambda: None):
+    def perform_request(self, endpoint, read_callback, url_params=None, data=None, raw_data=b"", method='GET',
+                        capture_errors=True, priority=QueuePriorityEnum.CRITICAL, on_cancel=lambda: None):
         """
         Perform a HTTP request.
         :param endpoint: the endpoint to call (i.e. "statistics")
         :param read_callback: the callback to be called with result info when we have the data
+        :param url_params: an optional dictionary with parameters that should be included in the URL
         :param data: optional POST data to be sent with the request
+        :param raw_data: optional raw data to include in the request, will get priority over data if defined
         :param method: the HTTP verb (GET/POST/PUT/PATCH)
         :param capture_errors: whether errors should be handled by this class (defaults to True)
+        :param priority: the priority of this request
+        :param on_cancel: optional callback to invoke when the request has been cancelled
         """
         self.on_cancel = on_cancel
 
         if read_callback:
             self.received_json.connect(read_callback)
 
+        url = endpoint + (("?" + tribler_urlencode(url_params)) if url_params else "")
+
+        if data and not raw_data:
+            data = tribler_urlencode(data)
+        elif raw_data:
+            data = raw_data
+
         def reply_callback(reply, log):
             log[-1] = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute)
             self.on_finished(reply, capture_errors)
 
-        request_queue.enqueue(self, method, endpoint, data, reply_callback, priority)
+        request_queue.enqueue(self, method, url, data, reply_callback, priority)
 
     @staticmethod
     def get_message_from_error(error):
         return_error = None
-        if isinstance(error['error'], (str, unicode)):
+        if isinstance(error['error'], string_types):
             return_error = error['error']
         elif 'message' in error['error']:
             return_error = error['error']['message']
@@ -288,9 +327,10 @@ class TriblerRequestManager(QObject):
 
         data = reply.readAll()
         try:
-            json_result = json.loads(str(data), encoding='latin_1')
+            json_result = json.loads(bytes(data), encoding='latin_1')
 
-            if 'error' in json_result and capture_errors:
+            if 'error' in json_result and capture_errors \
+                    and not TriblerRequestManager.window.core_manager.shutting_down:
                 self.show_error(TriblerRequestManager.get_message_from_error(json_result))
             else:
                 self.received_json.emit(json_result, reply.error())
@@ -382,6 +422,9 @@ class TriblerRequestWorker(QNetworkAccessManager):
         else:
             url = self.get_base_url() + endpoint
 
+        if data and not issubclass(type(data), bytes):
+            data = data.encode('utf8')
+
         log = [endpoint, method, data, time(), 0]
         performed_requests.append(log)
         network_reply = self.dispatch_map.get(method, lambda x, y, z: None)(endpoint, data, url)
@@ -398,10 +441,11 @@ class TriblerRequestWorker(QNetworkAccessManager):
         :param url: the url to send the request to.
         """
         buf = QBuffer()
-        buf.setData(data)
+        if data:
+            buf.setData(data)
         buf.open(QIODevice.ReadOnly)
         get_request = QNetworkRequest(QUrl(url))
-        reply = self.sendCustomRequest(get_request, "GET", buf)
+        reply = self.sendCustomRequest(get_request, b"GET", buf)
         buf.setParent(reply)
         return reply
 
@@ -417,7 +461,7 @@ class TriblerRequestWorker(QNetworkAccessManager):
         buf.setData(data)
         buf.open(QIODevice.ReadOnly)
         patch_request = QNetworkRequest(QUrl(url))
-        reply = self.sendCustomRequest(patch_request, "PATCH", buf)
+        reply = self.sendCustomRequest(patch_request, b"PATCH", buf)
         buf.setParent(reply)
         return reply
 
@@ -446,7 +490,7 @@ class TriblerRequestWorker(QNetworkAccessManager):
         buf.setData(data)
         buf.open(QIODevice.ReadOnly)
         delete_request = QNetworkRequest(QUrl(url))
-        reply = self.sendCustomRequest(delete_request, "DELETE", buf)
+        reply = self.sendCustomRequest(delete_request, b"DELETE", buf)
         buf.setParent(reply)
         return reply
 
