@@ -85,12 +85,11 @@ class EditChannelPage(QWidget):
     def showEvent(self, _event):
         self.update_channel_commit_views()
 
-    def update_channel_commit_views(self, deleted_index=None):
+    def update_channel_commit_views(self, reload_view=False):
         if self.channel_dirty and self.autocommit_enabled:
             self.commit_timer.stop()
             self.commit_timer.start(CHANNEL_COMMIT_DELAY)
-            if deleted_index:
-                # TODO: instead of reloading the whole table, just remove the deleted row and update start and end
+            if reload_view:
                 self.load_my_torrents()
 
         self.window().commit_control_bar.setHidden(not self.channel_dirty or self.autocommit_enabled)
