@@ -1,8 +1,6 @@
 from __future__ import absolute_import, division
 
 import os
-import random
-from binascii import hexlify
 from datetime import datetime
 
 from ipv8.database import database_blob
@@ -19,7 +17,7 @@ from Tribler.Core.Modules.MetadataStore.OrmBindings.channel_node import (
 from Tribler.Core.Modules.MetadataStore.OrmBindings.torrent_metadata import tdef_to_metadata_dict
 from Tribler.Core.Modules.MetadataStore.serialization import CHANNEL_TORRENT, ChannelMetadataPayload, REGULAR_TORRENT
 from Tribler.Core.TorrentDef import TorrentDef
-from Tribler.Core.Utilities.unicode import ensure_unicode
+from Tribler.Core.Utilities.unicode import ensure_unicode, hexlify
 from Tribler.Core.exceptions import DuplicateChannelIdError, DuplicateTorrentFileError
 
 CHANNEL_DIR_NAME_PK_LENGTH = 32  # Its not 40 so it could be distinguished from infohash
@@ -68,8 +66,7 @@ def entries_to_chunk(metadata_list, chunk_size, start_index=0):
         offset = len(header)
         out_list = [header]  # LZ4 header
         for index, metadata in enumerate(metadata_list[start_index:], start_index):
-            blob = c.compress(
-                ''.join(metadata.serialized_delete() if metadata.status == TODELETE else metadata.serialized()))
+            blob = c.compress((metadata.serialized_delete() if metadata.status == TODELETE else metadata.serialized()))
             # Chunk size limit reached?
             if offset + len(blob) > (chunk_size - LZ4_END_MARK_SIZE):
                 break
@@ -79,10 +76,10 @@ def entries_to_chunk(metadata_list, chunk_size, start_index=0):
             out_list.append(blob)
         out_list.append(c.flush())  # LZ4 end mark
 
-    chunk = ''.join(out_list)
+    chunk = b''.join(out_list)
     if last_entry_index is None:
         raise Exception('Serialized entry size > blob size limit!',
-                        hexlify(str(metadata_list[start_index].signature)))
+                        hexlify(metadata_list[start_index].signature))
     return chunk, last_entry_index + 1
 
 
@@ -139,7 +136,7 @@ def define_binding(db):
 
             my_channel = cls(origin_id=0, public_key=database_blob(cls._my_key.pub().key_to_bin()[10:]),
                              title=title, tags=description, subscribed=True, share=True, status=NEW,
-                             infohash=str(random.getrandbits(160)))
+                             infohash=os.urandom(20))
             # random infohash is necessary to avoid triggering DB uniqueness constraints
             my_channel.sign()
             return my_channel
@@ -204,7 +201,7 @@ def define_binding(db):
             # Make torrent out of dir with metadata files
             torrent, infohash = create_torrent_from_dir(channel_dir,
                                                         os.path.join(self._channels_dir, self.dirname + ".torrent"))
-            torrent_date = datetime.utcfromtimestamp(torrent['creation date'])
+            torrent_date = datetime.utcfromtimestamp(torrent[b'creation date'])
 
             return {"infohash": infohash, "num_entries": self.contents_len,
                     "timestamp": final_timestamp, "torrent_date": torrent_date}, torrent
@@ -232,7 +229,7 @@ def define_binding(db):
             except IOError:
                 self._logger.error(
                     "Error during channel torrent commit, not going to garbage collect the channel. Channel %s",
-                    hexlify(str(self.public_key)))
+                    hexlify(self.public_key))
             else:
                 if new_start_timestamp:
                     update_dict['start_timestamp'] = new_start_timestamp
@@ -251,7 +248,7 @@ def define_binding(db):
                 self.to_file(os.path.join(self._channels_dir, self.dirname + BLOB_EXTENSION))
 
                 self._logger.info("Channel %s committed with %i new entries. New version is %i",
-                                  hexlify(str(self.public_key)), len(md_list), update_dict['timestamp'])
+                                  hexlify(self.public_key), len(md_list), update_dict['timestamp'])
             return torrent
 
         @db_session
@@ -331,7 +328,7 @@ def define_binding(db):
         @property
         def dirname(self):
             # Have to limit this to support Windows file path length limit
-            return hexlify(self.public_key)[:CHANNEL_DIR_NAME_PK_LENGTH]+"{:0>16x}".format(self.id_)
+            return hexlify(self.public_key)[:CHANNEL_DIR_NAME_PK_LENGTH] + "{:0>16x}".format(self.id_)
 
         @property
         @db_session
@@ -426,7 +423,9 @@ def define_binding(db):
         @classmethod
         @db_session
         def get_updated_channels(cls):
-            return select(g for g in cls if g.subscribed and (g.local_version < g.timestamp) and
+            return select(g for g in cls if g.subscribed and
+                          g.status != LEGACY_ENTRY and
+                          (g.local_version < g.timestamp) and
                           g.public_key != database_blob(cls._my_key.pub().key_to_bin()[10:]))
 
         @classmethod
