@@ -5,16 +5,16 @@ import codecs
 import json
 import logging
 import os
-from binascii import hexlify, unhexlify
+from binascii import unhexlify
 
 from ipv8.database import database_blob
 
 from pony.orm import db_session
 
-from six import viewitems
 from six.moves.urllib.parse import unquote
 
 from twisted.internet.defer import Deferred
+from twisted.internet.error import ConnectionDone
 from twisted.web import http, resource
 from twisted.web.error import SchemeNotSupported
 from twisted.web.server import NOT_DONE_YET
@@ -23,8 +23,9 @@ import Tribler.Core.Utilities.json_util as json
 from Tribler.Core.Modules.MetadataStore.OrmBindings.channel_metadata import entries_to_chunk
 from Tribler.Core.Modules.restapi.metadata_endpoint import SpecificChannelTorrentsEndpoint
 from Tribler.Core.TorrentDef import TorrentDef
+from Tribler.Core.Utilities.unicode import hexlify, recursive_unicode
 from Tribler.Core.Utilities.utilities import http_get, is_infohash, parse_magnetlink
-from Tribler.Core.exceptions import DuplicateTorrentFileError
+from Tribler.Core.exceptions import DuplicateTorrentFileError, HttpError
 from Tribler.community.gigachannel.community import max_entries, maximum_payload_size
 
 
@@ -42,7 +43,7 @@ class BaseMyChannelEndpoint(SpecificChannelTorrentsEndpoint):
             u"error": {
                 u"handled": True,
                 u"code": exception.__class__.__name__,
-                u"message": exception.message
+                u"message": str(exception)
             }
         })
 
@@ -72,7 +73,7 @@ class MyChannelEndpoint(BaseMyChannelEndpoint):
             })
 
     def render_POST(self, request):
-        parameters = http.parse_qs(request.content.read(), 1)
+        parameters = recursive_unicode(http.parse_qs(request.content.read(), 1))
         if 'name' not in parameters and 'description' not in parameters:
             request.setResponseCode(http.BAD_REQUEST)
             return json.twisted_dumps({"error": "name or description parameter missing"})
@@ -84,15 +85,14 @@ class MyChannelEndpoint(BaseMyChannelEndpoint):
                 return json.twisted_dumps({"error": "your channel has not been created"})
 
             my_channel.update_metadata(update_dict={
-                "tags": unquote(parameters['description'][0]).decode('utf-8'),
-                "title": unquote(parameters['name'][0]).decode('utf-8')
+                "tags": unquote(parameters['description'][0]),
+                "title": unquote(parameters['name'][0])
             })
 
         return json.twisted_dumps({"edited": True})
 
     def render_PUT(self, request):
-        parameters = http.parse_qs(request.content.read(), 1)
-
+        parameters = recursive_unicode(http.parse_qs(request.content.read(), 1))
         if 'name' not in parameters or not parameters['name'] or not parameters['name'][0]:
             request.setResponseCode(http.BAD_REQUEST)
             return json.twisted_dumps({"error": "channel name cannot be empty"})
@@ -100,7 +100,7 @@ class MyChannelEndpoint(BaseMyChannelEndpoint):
         if 'description' not in parameters or not parameters['description']:
             description = u''
         else:
-            description = unquote(parameters['description'][0]).decode('utf-8')
+            description = unquote(parameters['description'][0])
 
         my_key = self.session.trustchain_keypair
         my_channel_pk = my_key.pub().key_to_bin()
@@ -110,10 +110,10 @@ class MyChannelEndpoint(BaseMyChannelEndpoint):
             request.setResponseCode(http.CONFLICT)
             return json.twisted_dumps({"error": "channel already exists"})
 
-        title = unquote(parameters['name'][0]).decode('utf-8')
+        title = unquote(parameters['name'][0])
         self.session.lm.mds.ChannelMetadata.create_channel(title, description)
         return json.twisted_dumps({
-            "added": hexlify(str(my_channel_pk)),
+            "added": hexlify(my_channel_pk),
         })
 
 
@@ -133,7 +133,8 @@ class SpecificChannelExportEndpoint(BaseMyChannelEndpoint):
             serialized_data = entries_to_chunk([my_channel] + random_channel_torrents, maximum_payload_size)[0]
 
         request.setHeader(b'content-type', 'application/x-bittorrent')
-        request.setHeader(b'Content-Disposition', 'attachment; filename=%s.mdblob.lz4' % hexlify(my_channel.public_key))
+        request.setHeader(b'Content-Disposition', 'attachment; filename=%s.mdblob.lz4'
+                          % hexlify(my_channel.public_key).encode('utf-8'))
         return serialized_data
 
 
@@ -151,9 +152,10 @@ class MyChannelTorrentsEndpoint(BaseMyChannelEndpoint):
                 request.setResponseCode(http.NOT_FOUND)
                 return json.twisted_dumps({"error": "your channel has not been created"})
 
-            sanitized = self.sanitize_parameters(request.args)
-            if b'exclude_deleted' in request.args:
-                sanitized['exclude_deleted'] = bool(int(request.args[b'exclude_deleted'][0]) > 0)
+            args = recursive_unicode(request.args)
+            sanitized = self.sanitize_parameters(args)
+            if 'exclude_deleted' in args:
+                sanitized['exclude_deleted'] = args['exclude_deleted']
 
             sanitized.update(dict(channel_pk=database_blob(my_channel.public_key)))
 
@@ -169,7 +171,7 @@ class MyChannelTorrentsEndpoint(BaseMyChannelEndpoint):
             })
 
     def render_POST(self, request):
-        parameters = http.parse_qs(request.content.read(), 1)
+        parameters = recursive_unicode(http.parse_qs(request.content.read(), 1))
         if 'status' not in parameters or 'infohashes' not in parameters:
             request.setResponseCode(http.BAD_REQUEST)
             return json.twisted_dumps({"error": "status or infohashes parameter missing"})
@@ -251,7 +253,7 @@ class MyChannelTorrentsEndpoint(BaseMyChannelEndpoint):
                 request.setResponseCode(http.NOT_FOUND)
                 return json.twisted_dumps({"error": "your channel has not been created yet"})
 
-        parameters = http.parse_qs(request.content.read(), 1)
+        parameters = recursive_unicode(http.parse_qs(request.content.read(), 1))
 
         if 'description' not in parameters or not parameters['description']:
             extra_info = {}
@@ -260,13 +262,27 @@ class MyChannelTorrentsEndpoint(BaseMyChannelEndpoint):
 
         title = parameters['title'][0] if 'title' in parameters and parameters['title'] else None
 
+        # This is required to determine if the connection can be closed by the server
+        can_close = [True]
+
+        def _faulty_req_termination(failure, can_close):
+            # If this callback was triggered, then the connection is guaranteed to have been closed.
+            can_close[0] = False
+            if failure is not None:
+                failure.trap(ConnectionDone)
+                self._logger.exception("Connection did not close properly: %s %s", failure.getErrorMessage(),
+                                       failure.type)
+
+        request.notifyFinish().addBoth(_faulty_req_termination, can_close)
+
         def _on_url_fetched(data):
             return TorrentDef.load_from_memory(data)
 
         def _on_magnet_fetched(meta_info):
             if not meta_info:
                 request.write(self.return_500(request, RuntimeError("Metainfo timeout")))
-                request.finish()
+                if can_close[0]:
+                    request.finish()
                 return
 
             return TorrentDef.load_from_dict(meta_info)
@@ -282,13 +298,15 @@ class MyChannelTorrentsEndpoint(BaseMyChannelEndpoint):
 
         def _on_added(added):
             request.write(json.twisted_dumps({"added": added}))
-            request.finish()
+            if can_close[0]:
+                request.finish()
 
         def _on_add_failed(failure):
-            failure.trap(ValueError, DuplicateTorrentFileError, SchemeNotSupported)
+            failure.trap(ValueError, DuplicateTorrentFileError, SchemeNotSupported, HttpError)
             self._logger.exception(failure.value)
             request.write(self.return_500(request, failure.value))
-            request.finish()
+            if can_close[0]:
+                request.finish()
 
         # First, check whether we did upload a magnet link or URL
         if 'uri' in parameters and parameters['uri']:
@@ -315,14 +333,14 @@ class MyChannelTorrentsEndpoint(BaseMyChannelEndpoint):
             return NOT_DONE_YET
 
         torrents_dir = None
-        if 'torrents_dir' in parameters and parameters['torrents_dir'] > 0:
+        if 'torrents_dir' in parameters and parameters['torrents_dir']:
             torrents_dir = parameters['torrents_dir'][0]
             if not os.path.isabs(torrents_dir):
                 request.setResponseCode(http.BAD_REQUEST)
                 return json.twisted_dumps({"error": "the torrents_dir should point to a directory"})
 
         recursive = False
-        if 'recursive' in parameters and parameters['recursive'] > 0:
+        if 'recursive' in parameters and parameters['recursive']:
             recursive = parameters['recursive'][0]
             if not torrents_dir:
                 request.setResponseCode(http.BAD_REQUEST)
@@ -363,9 +381,10 @@ class MyChannelTorrentsCountEndpoint(BaseMyChannelEndpoint):
                 request.setResponseCode(http.NOT_FOUND)
                 return json.twisted_dumps({"error": "your channel has not been created"})
 
-            sanitized = self.sanitize_parameters(request.args)
-            if b'exclude_deleted' in request.args:
-                sanitized['exclude_deleted'] = request.args[b'exclude_deleted']
+            args = recursive_unicode(request.args)
+            sanitized = self.sanitize_parameters(args)
+            if 'exclude_deleted' in args:
+                sanitized['exclude_deleted'] = args['exclude_deleted']
 
             sanitized.update(dict(channel_pk=database_blob(my_channel.public_key)))
             return self.get_total_count(self.session.lm.mds.TorrentMetadata, sanitized)
@@ -404,12 +423,7 @@ class MyChannelSpecificTorrentEndpoint(BaseMyChannelEndpoint):
             :statuscode 404: if your channel or the infohash does not exist.
             :statuscode 500: if the passed arguments data is wrong.
         """
-        parameters_raw = http.parse_qs(request.content.read(), 1)
-        parameters = {}
-        # FIXME: make all endpoints Unicode-compatible in a unified way
-        for param, val in viewitems(parameters_raw):
-            parameters.update({param: [item.decode('utf-8') for item in val]})
-
+        parameters = recursive_unicode(http.parse_qs(request.content.read(), 1))
         if 'status' not in parameters and 'tags' not in parameters and 'title' not in parameters:
             request.setResponseCode(http.BAD_REQUEST)
             return json.twisted_dumps({"error": "attribute to change is missing"})
