@@ -1,10 +1,13 @@
+from __future__ import absolute_import
+
 import logging
 import os
+
 from twisted.internet import reactor, task
 from twisted.web import resource
 
-from Tribler.Core.Modules.process_checker import ProcessChecker
 import Tribler.Core.Utilities.json_util as json
+from Tribler.Core.Modules.process_checker import ProcessChecker
 
 
 class ShutdownEndpoint(resource.Resource):
@@ -42,6 +45,9 @@ class ShutdownEndpoint(resource.Resource):
             reactor.addSystemEventTrigger('after', 'shutdown', os._exit, code)
             reactor.stop()
             self.process_checker.remove_lock_file()
+            # Flush the logs to the file before exiting
+            for handler in logging.getLogger().handlers:
+                handler.flush()
 
         def log_and_shutdown(failure):
             self._logger.error(failure.value)
@@ -51,4 +57,4 @@ class ShutdownEndpoint(resource.Resource):
             .addCallback(shutdown_process) \
             .addErrback(log_and_shutdown)
 
-        return json.dumps({"shutdown": True})
+        return json.twisted_dumps({"shutdown": True})
