@@ -32,9 +32,8 @@ DOWNLOAD = 'down'
 DLMODE_NORMAL = 0
 DLMODE_VOD = 1
 
-PERSISTENTSTATE_CURRENTVERSION = 5
 
-STATEDIR_DLPSTATE_DIR = u'dlcheckpoints'
+STATEDIR_CHECKPOINT_DIR = u'dlcheckpoints'
 STATEDIR_WALLET_DIR = u'wallet'
 STATEDIR_CHANNELS_DIR = u'channels'
 STATEDIR_DB_DIR = u"sqlite"
