@@ -47,7 +47,7 @@ class TrustChainCommunity(Community):
     master_peer = Peer(unhexlify("4c69624e61434c504b3a5730f52156615ecbcedb36c442992ea8d3c26b418edd8bd00e01dce26028cd"
                                  "1ebe5f7dce59f4ed59f8fcee268fd7f1c6dc2fa2af8c22e3170e00cdecca487745"))
 
-    UNIVERSAL_BLOCK_LISTENER = 'UNIVERSAL_BLOCK_LISTENER'
+    UNIVERSAL_BLOCK_LISTENER = b'UNIVERSAL_BLOCK_LISTENER'
     DB_CLASS = TrustChainDB
     DB_NAME = 'trustchain'
     version = b'\x02'
@@ -413,7 +413,7 @@ class TrustChainCommunity(Community):
                 return self.sign_block(peer, linked=blk)
 
         # determine if we want to sign this block
-        return addCallback(self.should_sign(blk), on_should_sign_outcome)
+        return addCallback(maybeDeferred(self.should_sign, blk), on_should_sign_outcome)
 
     def crawl_chain(self, peer, latest_block_num=0):
         """
@@ -543,7 +543,8 @@ class TrustChainCommunity(Community):
             end_seq_num = max(GENESIS_SEQ, last_block.sequence_number + end_seq_num + 1) \
                 if last_block else GENESIS_SEQ
 
-        blocks = self.persistence.crawl(payload.public_key, start_seq_num, end_seq_num, limit=10)
+        blocks = self.persistence.crawl(payload.public_key, start_seq_num, end_seq_num,
+                                        limit=self.settings.max_crawl_batch)
         total_count = len(blocks)
 
         if total_count == 0:
