@@ -2,13 +2,21 @@ from __future__ import absolute_import
 
 from hashlib import sha256, sha512
 
+from six import int2byte, text_type
+
+
+def to_ascii(value):
+    if isinstance(value, text_type):
+        return b''.join(int2byte(ord(c)) for c in value)
+    return value
+
 
 def sha512_as_int(value):
     """
     Convert a SHA512 hash to an integer.
     """
     out = 0
-    hashed = sha512(value).digest()
+    hashed = sha512(to_ascii(value)).digest()
     for i in range(len(hashed)):
         out <<= 8
         out |= ord(hashed[i:i + 1])
@@ -20,7 +28,7 @@ def sha256_as_int(value):
     Convert a SHA256 hash to an integer.
     """
     out = 0
-    hashed = sha256(value).digest()
+    hashed = sha256(to_ascii(value)).digest()
     for i in range(len(hashed)):
         out <<= 8
         out |= ord(hashed[i:i + 1])
@@ -32,7 +40,7 @@ def sha256_4_as_int(value):
     Convert a SHA256 4 byte hash to an integer.
     """
     out = 0
-    hashed = sha256(value).digest()[:4]
+    hashed = sha256(to_ascii(value)).digest()[:4]
     for i in range(len(hashed)):
         out <<= 8
         out |= ord(hashed[i:i + 1])
