@@ -1,30 +1,28 @@
 from __future__ import absolute_import
 from __future__ import division
 
-from binascii import hexlify, unhexlify
+import hashlib
 import os
 import time
-import hashlib
+from binascii import hexlify, unhexlify
+from collections import defaultdict, deque
 
-from collections import deque, defaultdict
+import six
 
-from twisted.internet.defer import inlineCallbacks, Deferred, fail, DeferredList, returnValue
+from twisted.internet.defer import Deferred, DeferredList, fail, inlineCallbacks, returnValue
 from twisted.internet.task import LoopingCall
 from twisted.python.failure import Failure
 
-from ..peer import Peer
-from ..requestcache import RandomNumberCache, RequestCache
-from ..peerdiscovery.churn import PingChurn
-from ..messaging.payload import PuncturePayload
-from ..messaging.payload_headers import BinMemberAuthenticationPayload, GlobalTimeDistributionPayload
+from .payload import (FindRequestPayload, FindResponsePayload, PingRequestPayload, PingResponsePayload,
+                      SignedStrPayload, StoreRequestPayload, StoreResponsePayload, StrPayload)
+from .routing import Node, RoutingTable, calc_node_id, distance
+from .storage import Storage
 from ..community import Community
 from ..lazy_community import lazy_wrapper, lazy_wrapper_wd
-
-from .storage import Storage
-from .routing import RoutingTable, Node, distance, calc_node_id
-from .payload import PingRequestPayload, PingResponsePayload, StoreRequestPayload, \
-                     StoreResponsePayload, FindRequestPayload, FindResponsePayload, \
-                     SignedStrPayload, StrPayload
+from ..messaging.payload_headers import BinMemberAuthenticationPayload, GlobalTimeDistributionPayload
+from ..peer import Peer
+from ..peerdiscovery.churn import PingChurn
+from ..requestcache import RandomNumberCache, RequestCache
 from ..util import addCallback, cast_to_bin
 
 PING_INTERVAL = 25
@@ -35,14 +33,14 @@ TOKEN_EXPIRATION_TIME = 600
 DHT_ENTRY_STR = 0
 DHT_ENTRY_STR_SIGNED = 1
 
-MAX_ENTRY_SIZE = 155
+MAX_ENTRY_SIZE = 170
 MAX_ENTRY_AGE = 86400
 
 MAX_FIND_WALKS = 8
 MAX_FIND_STEPS = 4
 
-MAX_VALUES_IN_STORE = 9
-MAX_VALUES_IN_FIND = 9
+MAX_VALUES_IN_STORE = 8
+MAX_VALUES_IN_FIND = 8
 MAX_NODES_IN_FIND = 8
 
 # Target number of nodes at which a key-value pair should be stored
@@ -81,7 +79,7 @@ class Request(RandomNumberCache):
 
     def on_timeout(self):
         if not self.deferred.called:
-            self._logger.info('Timeout for %s to %s', self.msg_type, self.node)
+            self._logger.debug('Timeout for %s to %s', self.msg_type, self.node)
             self.node.failed += 1
             if not self.consume_errors:
                 self.deferred.errback(Failure(RuntimeError('Timeout for {} to {}'.format(self.msg_type, self.node))))
@@ -96,11 +94,8 @@ class DHTCommunity(Community):
     """
     Community for storing/finding key-value pairs.
     """
-    master_peer = Peer(unhexlify('3081a7301006072a8648ce3d020106052b81040027038192000402b8dcc3bee358ff30b3bd00aa56a7f'
-                                 '9100fe546fc7fb518f197f7c1bbadeb817d8e7a0dcc44c239cfc8906cf55781235dfe24179d0450bcdb'
-                                 'dc49e851f3b03b12acbf0e7b0823e606e50e9c0c77a97af7e737ad749ff53c42fd9d636da4e71d3b7a7'
-                                 'a291734fd64304be90c86e2268490332dfa9e8f01d953538f01c2fba73acd7fe7eec480098054610855'
-                                 'b7918974'))
+    master_peer = Peer(unhexlify('4c69624e61434c504b3a4c99f04cef9ba4ca645401cd51b8ef634e63e2ad0d3209eca958ce0293d7cf668'
+                                 '2059469c1a253e66191bd3b96a082a8e11cc35962b9b6f8434e21518a0344af'))
 
     def __init__(self, *args, **kwargs):
         super(DHTCommunity, self).__init__(*args, **kwargs)
@@ -160,13 +155,6 @@ class DHTCommunity(Community):
     def introduction_response_callback(self, peer, dist, payload):
         self.on_node_discovered(peer.public_key.key_to_bin(), peer.address)
 
-    @lazy_wrapper(GlobalTimeDistributionPayload, PuncturePayload)
-    def on_puncture(self, peer, dist, payload):
-        # Since the DHT sends punctures manually, we often discover peers that the
-        # DiscoveryStrategy did not ask for. In order to keep these peers from entering
-        # our list of verified peers, we ignore punctures.
-        pass
-
     def on_node_discovered(self, public_key_bin, source_address):
         # Filter out trackers
         if source_address not in self.network.blacklist:
@@ -255,7 +243,7 @@ class DHTCommunity(Community):
         # Check if we also need to store this key-value pair
         largest_distance = max([distance(node.id, key) for node in nodes])
         if distance(self.my_node_id, key) < largest_distance:
-            for value in values:
+            for value in reversed(values):
                 self.add_value(key, value)
 
         now = time.time()
@@ -269,8 +257,8 @@ class DHTCommunity(Community):
             else:
                 self.logger.debug('Not sending store-request to %s (no token available)', node)
 
-        return gatherResponses(deferreds, consumeErrors=True) \
-               if deferreds else fail(RuntimeError('Value was not stored'))
+        return (gatherResponses(deferreds, consumeErrors=True) if deferreds
+                else fail(RuntimeError('Value was not stored')))
 
     @lazy_wrapper(GlobalTimeDistributionPayload, StoreRequestPayload)
     def on_store_request(self, peer, dist, payload):
@@ -316,20 +304,20 @@ class DHTCommunity(Community):
         cache.on_complete()
         cache.deferred.callback(cache.node)
 
-    def _send_find_request(self, node, target, force_nodes):
+    def _send_find_request(self, node, target, force_nodes, start_idx=0):
         cache = self.request_cache.add(Request(self, u'find', node, [force_nodes]))
         self.send_message(node.address, MSG_FIND_REQUEST, FindRequestPayload,
-                          (cache.number, self.my_estimated_lan, target, force_nodes))
+                          (cache.number, self.my_estimated_lan, target, start_idx, force_nodes))
         return cache.deferred
 
     def _process_find_responses(self, responses, nodes_tried):
-        values = set()
+        values = []
         nodes = set()
         to_puncture = {}
 
         for sender, response in responses:
             if 'values' in response:
-                values |= set(response['values'])
+                values.append(response['values'])
             else:
                 # Pick a node that we haven't tried yet.
                 node = next((n for n in response['nodes']
@@ -345,13 +333,15 @@ class DHTCommunity(Community):
         return values, nodes, to_puncture
 
     @inlineCallbacks
-    def _find(self, target, force_nodes=False):
+    def _find(self, target, force_nodes=False, start_idx=0):
+        assert start_idx >= 0, "The query cannot start from a negative index"
+
         nodes_closest = set(self.routing_table.closest_nodes(target, max_nodes=MAX_FIND_WALKS))
         if not nodes_closest:
             returnValue(Failure(RuntimeError('No nodes found in the routing table')))
 
         nodes_tried = set()
-        values = set()
+        values = []
         recent = None
 
         for _ in range(MAX_FIND_STEPS):
@@ -359,7 +349,8 @@ class DHTCommunity(Community):
                 break
 
             # Send closest nodes a find-node-request
-            deferreds = [self._send_find_request(node, target, force_nodes) for node in nodes_closest]
+            deferreds = [self._send_find_request(node, target, force_nodes, start_idx=start_idx)
+                         for node in nodes_closest]
             responses = yield gatherResponses(deferreds, consumeErrors=True)
             recent = next((sender for sender, response in responses if 'nodes' in response), recent)
 
@@ -368,7 +359,7 @@ class DHTCommunity(Community):
 
             # Process responses and puncture nodes that we haven't tried yet
             new_values, new_nodes, to_puncture = self._process_find_responses(responses, nodes_tried)
-            values |= new_values
+            values += new_values
             nodes_closest |= new_nodes
 
             deferreds = [self._send_find_request(sender, node.id, force_nodes)
@@ -386,7 +377,12 @@ class DHTCommunity(Community):
         if force_nodes:
             returnValue(sorted(nodes_tried, key=lambda n: distance(n.id, target)))
 
-        values = list(values)
+        # Merge all values received into one tuple. First pick the first value from each tuple, then the second, etc.
+        values = sum(six.moves.zip_longest(*values), ())
+
+        # Filter out duplicates while preserving order
+        seen = set()
+        values = [v for v in values if v is not None and not (v in seen or seen.add(v))]
 
         if recent and values:
             # Store the key-value pair on the most recently visited node that
@@ -417,8 +413,8 @@ class DHTCommunity(Community):
 
         return results
 
-    def find_values(self, target):
-        return self._find(target, force_nodes=False)
+    def find_values(self, target, start_idx=0):
+        return self._find(target, force_nodes=False, start_idx=start_idx)
 
     def find_nodes(self, target):
         return self._find(target, force_nodes=True)
@@ -432,7 +428,8 @@ class DHTCommunity(Community):
             return
 
         nodes = []
-        values = self.storage.get(payload.target, limit=MAX_VALUES_IN_FIND) if not payload.force_nodes else []
+        values = self.storage.get(payload.target, starting_point=payload.start_idx, limit=MAX_VALUES_IN_FIND) \
+            if not payload.force_nodes else []
 
         if payload.force_nodes or not values:
             nodes = self.routing_table.closest_nodes(payload.target, exclude_node=node, max_nodes=MAX_NODES_IN_FIND)
@@ -462,8 +459,8 @@ class DHTCommunity(Community):
         elif cache.params[0]:
             cache.deferred.callback((cache.node, {'nodes': payload.nodes}))
         else:
-            cache.deferred.callback((cache.node, {'values': payload.values} if payload.values else \
-                                                 {'nodes': payload.nodes}))
+            cache.deferred.callback((cache.node, {'values': payload.values} if payload.values
+                                     else {'nodes': payload.nodes}))
 
     def value_maintenance(self):
         # Refresh buckets
@@ -484,7 +481,7 @@ class DHTCommunity(Community):
 
         # Cleanup old tokens
         now = time.time()
-        for node, (ts, _) in self.tokens.items():
+        for node, (ts, _) in list(self.tokens.items()):
             if now > ts + TOKEN_EXPIRATION_TIME:
                 self.tokens.pop(node, None)
 
