@@ -1,6 +1,6 @@
 from __future__ import absolute_import
 
-from setuptools import setup, find_packages
+from setuptools import find_packages, setup
 
 with open("README.md", "r") as fh:
     long_description = fh.read()
@@ -11,7 +11,7 @@ setup(
     description='The Python implementation of the IPV8 library',
     long_description=long_description,
     long_description_content_type='text/markdown',
-    version='1.5.0',
+    version='1.10.0',
     url='https://github.com/Tribler/py-ipv8',
     package_data={'': ['*.*']},
     packages=find_packages(),
@@ -22,8 +22,12 @@ setup(
         "netifaces",
         "Twisted",
         "pyOpenSSL",
-        "six"
+        "six",
+        "pyasn1"
     ],
+    extras_require={
+        'netifaces': ["netifaces"]
+    },
     classifiers=[
         "Development Status :: 5 - Production/Stable",
         "Framework :: Twisted",
