@@ -154,3 +154,10 @@ BITTORRENT_BIRTHDAY = 994032000
 # Timeout for metainfo request
 METAINFO_MAX_RETRIES = 3
 METAINFO_TIMEOUT = 30000
+
+# Sizes
+KB = 1024
+MB = 1024 * KB
+GB = 1024 * MB
+TB = 1024 * GB
+PB = 1024 * TB
