@@ -8,29 +8,64 @@ import sys
 import time
 import traceback
 from base64 import b64encode
-from binascii import hexlify
-
 
 from PyQt5 import uic
 from PyQt5.QtCore import (
-    QCoreApplication, QDir, QObject, QPoint, QSettings, QStringListModel, QTimer, QUrl, Qt, pyqtSignal, pyqtSlot)
+    QCoreApplication,
+    QDir,
+    QObject,
+    QPoint,
+    QSettings,
+    QStringListModel,
+    QTimer,
+    QUrl,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
 from PyQt5.QtGui import QDesktopServices, QIcon, QKeySequence, QPixmap
 from PyQt5.QtWidgets import (
-    QAction, QApplication, QCompleter, QFileDialog, QLineEdit, QListWidget, QMainWindow, QShortcut, QStyledItemDelegate,
-    QSystemTrayIcon, QTreeWidget)
+    QAction,
+    QApplication,
+    QCompleter,
+    QFileDialog,
+    QLineEdit,
+    QListWidget,
+    QMainWindow,
+    QShortcut,
+    QStyledItemDelegate,
+    QSystemTrayIcon,
+    QTreeWidget,
+)
 
 import six
 from six.moves.urllib.parse import unquote, urlparse
 
 from Tribler.Core.Modules.process_checker import ProcessChecker
+from Tribler.Core.Utilities.unicode import hexlify
 
 from TriblerGUI.core_manager import CoreManager
 from TriblerGUI.debug_window import DebugWindow
 from TriblerGUI.defs import (
-    BUTTON_TYPE_CONFIRM, BUTTON_TYPE_NORMAL, CONTEXT_MENU_WIDTH, DEFAULT_API_PORT, PAGE_CHANNEL_DETAILS,
-    PAGE_DISCOVERED, PAGE_DISCOVERING, PAGE_DOWNLOADS, PAGE_EDIT_CHANNEL, PAGE_HOME, PAGE_LOADING,
-    PAGE_SEARCH_RESULTS, PAGE_SETTINGS, PAGE_SUBSCRIBED_CHANNELS, PAGE_TRUST, PAGE_TRUST_GRAPH_PAGE,
-    PAGE_VIDEO_PLAYER, SHUTDOWN_WAITING_PERIOD)
+    BUTTON_TYPE_CONFIRM,
+    BUTTON_TYPE_NORMAL,
+    CONTEXT_MENU_WIDTH,
+    DEFAULT_API_PORT,
+    PAGE_CHANNEL_DETAILS,
+    PAGE_DISCOVERED,
+    PAGE_DISCOVERING,
+    PAGE_DOWNLOADS,
+    PAGE_EDIT_CHANNEL,
+    PAGE_HOME,
+    PAGE_LOADING,
+    PAGE_SEARCH_RESULTS,
+    PAGE_SETTINGS,
+    PAGE_SUBSCRIBED_CHANNELS,
+    PAGE_TRUST,
+    PAGE_TRUST_GRAPH_PAGE,
+    PAGE_VIDEO_PLAYER,
+    SHUTDOWN_WAITING_PERIOD,
+)
 from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
 from TriblerGUI.dialogs.createtorrentdialog import CreateTorrentDialog
 from TriblerGUI.dialogs.feedbackdialog import FeedbackDialog
@@ -98,7 +133,6 @@ class TriblerWindow(QMainWindow):
         QCoreApplication.setOrganizationDomain("nl")
         QCoreApplication.setOrganizationName("TUDelft")
         QCoreApplication.setApplicationName("Tribler")
-        QCoreApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)
 
         self.gui_settings = QSettings()
         api_port = api_port or int(get_gui_setting(self.gui_settings, "api_port", DEFAULT_API_PORT))
@@ -424,7 +458,7 @@ class TriblerWindow(QMainWindow):
                             "but you do not have one yet. No worries, you can easily create your own channel by " \
                             "navigating to My Channel section on the left sidebar."
             ConfirmationDialog.show_error(self.window(), "Download Error!", error_message)
-            return
+            add_to_channel = False
 
         selected_files_list = []
         if len(selected_files) != total_files:  # Not all files included
@@ -465,7 +499,7 @@ class TriblerWindow(QMainWindow):
         post_data = dict()
         if uri.startswith("file:"):
             with open(uri[5:], "rb") as torrent_file:
-                post_data['torrent'] = b64encode(torrent_file.read())
+                post_data['torrent'] = b64encode(torrent_file.read()).decode('utf8')
         elif uri.startswith("magnet:"):
             post_data['uri'] = uri
 
@@ -670,7 +704,7 @@ class TriblerWindow(QMainWindow):
         plus_btn_geometry = self.add_torrent_button.geometry()
         plus_btn_pos.setX(plus_btn_pos.x() - CONTEXT_MENU_WIDTH)
         plus_btn_pos.setY(plus_btn_pos.y() + plus_btn_geometry.height())
-        self.create_add_torrent_menu().exec_(self.mapToGlobal(plus_btn_pos))
+        self.create_add_torrent_menu().exec_(self.mapToParent(plus_btn_pos))
 
     def on_add_torrent_browse_file(self):
         filenames = QFileDialog.getOpenFileNames(self,
@@ -693,6 +727,7 @@ class TriblerWindow(QMainWindow):
             self.process_uri_request()
 
     def start_download_from_uri(self, uri):
+        uri = uri.decode('utf-8') if isinstance(uri, six.binary_type) else uri
         self.download_uri = uri
 
         if get_gui_setting(self.gui_settings, "ask_download_settings", True, is_bool=True):
