@@ -1,14 +1,14 @@
-import datetime
-from PyQt5.QtCore import Qt
-from PyQt5.QtCore import pyqtSignal
-from PyQt5.QtGui import QCursor
-from PyQt5.QtGui import QIcon
-from PyQt5.QtWidgets import QAction
-from PyQt5.QtWidgets import QSizePolicy
-from PyQt5.QtWidgets import QSpacerItem
-from PyQt5.QtWidgets import QWidget
+from __future__ import absolute_import
 
-from TriblerGUI.defs import PAGE_MARKET_TRANSACTIONS, PAGE_MARKET_WALLETS, PAGE_MARKET_ORDERS
+import datetime
+
+from PyQt5.QtCore import Qt, pyqtSignal
+from PyQt5.QtGui import QCursor, QIcon
+from PyQt5.QtWidgets import QAction, QSizePolicy, QSpacerItem, QWidget
+
+from six.moves import xrange
+
+from TriblerGUI.defs import PAGE_MARKET_ORDERS, PAGE_MARKET_TRANSACTIONS, PAGE_MARKET_WALLETS
 from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
 from TriblerGUI.dialogs.newmarketorderdialog import NewMarketOrderDialog
 from TriblerGUI.tribler_action_menu import TriblerActionMenu
@@ -99,7 +99,7 @@ class MarketPage(QWidget):
             self.chosen_wallets = sorted(self.wallets.keys())[0], sorted(self.wallets.keys())[1]
             self.update_button_texts()
 
-        for wallet_id, wallet in wallets.iteritems():
+        for wallet_id, wallet in wallets.items():
             if not wallet['created'] or not wallet['unlocked']:
                 continue
 
@@ -246,8 +246,12 @@ class MarketPage(QWidget):
         """
         Create a new ask or bid order.
         """
-        post_data = str("first_asset_amount=%d&first_asset_type=%s&second_asset_amount=%d&second_asset_type=%s" %
-                        (asset1_amount, asset1_type, asset2_amount, asset2_type))
+        post_data = {
+            "first_asset_amount": asset1_amount,
+            "first_asset_type": asset1_type,
+            "second_asset_amount": asset2_amount,
+            "second_asset_type": asset2_type
+        }
         self.request_mgr = TriblerRequestManager()
         self.request_mgr.perform_request("market/%s" % ('asks' if is_ask else 'bids'),
                                          lambda response: self.on_order_created(response, is_ask),
